/* BrowserFrame.java - hades.manager.BrowserFrame
 *
 * the main GUI for the Colibri design browser
 *
 * 13.03.01 - try t1-libraries.jar and t3-vorlagen.jar
 * 12.03.01 - added setThumbnail
 * 06.03.01 - minor layout changes
 * 05.03.01 - first menu actions 
 * 04.03.01 - delegate TreeSelection events to tree subclasses, vert. splitpane 
 * 03.03.01 - added inner class TreeSelectionHandler
 * 02.03.01 - first action handlers re-activated
 * 01.03.01 - new version, integration into Hades 0.87
 * 01.07.98 - original version by
 * @author    Ulrich Dallmann       (2dallman@informatik.uni-hamburg.de)
 * @author    Karola Kroenert       (2kroene@informatik.uni-hamburg.de
 */

package hades.manager;

import java.awt.*;
import java.awt.event.*;
import java.util.zip.*;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.text.html.HTMLEditorKit;

import jfig.gui.ImageHelper;
import hades.manager.tree.*;



/**
 * the toplevel GUI component for the Colibri design browser,
 * with the SimObject/Design-files tree on the left,
 * and image and HTML components on the right.
 */
public class  BrowserFrame 
       extends  JFrame
       implements  ActionListener
{
  public static boolean debug = true;
  public static String  versionString = "Colibri 0.42 (01.03.2001)";

  private TreeManager treeManager;
  
  /** used for cut and paste */
  public Object clipboard = null;
  
  //private  static Browser  browser = null;  // the singleton design browser
  
  
  /* Panels and GUI stuff
   *     sp    rp
   *  ---------------
   *  |      |      |
   *  |      | ic   |
   *  |  tp  |------|   
   *  |      |      |
   *  |      | ip   |
   *  ---------------
   */

  private JSplitPane     splitPane;             // vertical divider
  private JTree          tree;                  // the object tree
  private JScrollPane    treeScroller;
  //private JPanel       rightPanel;            // covers image and info panel
  private JSplitPane     rightPanel;            // covers image and info panel
  private JScrollPane    imageScroller;
  private JLabel         imageCanvas;           // to display the image
  private InfoPanel      infoPanel;             // author, version, help text



  // MenuBar, Menu and Menu item definitions
  JMenuBar mb;
  JMenu    mFile, mEdit, mBookmark, mSearch, mExtra, mHelp;


  JMenuItem miFolder;    // Folder
  JMenuItem miOptions;    // Options
  JMenuItem miBeanExp;    // BeanExplorer
  JMenuItem miExit;      // Exit
  JMenuItem miCopy;      // Copy
  JMenuItem miPaste;      // Paste
  JMenuItem miEditBM;    // Edit Bookmarks
  JMenuItem miRefresh;    // Refresh tree
  JMenuItem miLSearch;    // local Search
  JMenuItem miGSearch;    // global Search
  JMenuItem miPrintURL;   // print url
  JMenu      mDebug;      // debugging
  JMenuItem miShowConsole;// show console
  JMenuItem miCLB;        // Create clb
  JMenuItem miImages;    // Create images
  JMenuItem miHelp;      // Help
  JMenuItem miAbout;      // About


  
  /**
  *  
  *  
  */
  public BrowserFrame () {
    super( versionString );

    this.setBackground( Color.lightGray );

    setLookAndFeel();
    createMenuBar();
    createTree();
    createTreePopupMenu();
    createImagePanel();
    createInfoPanel();
    createSplitPaneAndLayout();

    this.addWindowListener( new WindowHandler() );
    tree.addTreeSelectionListener( new TreeSelectionHandler() );
    new TreePopupMenu( tree );
  }


  public void setTreeManager( TreeManager manager ) {
    treeManager = manager;
  }
    

 
  public void createMenuBar() { 
    if (debug) msg( "-#- createMenuBar..." );
    // Create menubar and attach to the frame
    mb = new JMenuBar();
    this.setJMenuBar(mb);
    // mb.setFont(new Font("SansSerif", Font.PLAIN, 12));
    // Create menus and assign to menubar
    mFile = new JMenu("File");
    mEdit = new JMenu("Edit");
    mBookmark = new JMenu("Bookmark");
    mSearch = new JMenu("Search");
    mExtra = new JMenu("Special");
    mHelp = new JMenu("Help");    

    mb.add(mFile);
    mb.add(mEdit);
    mb.add(mBookmark);
    mb.add(mSearch);
    mb.add(mExtra);
    mb.add(mHelp);
    
    // Create menu items and assign to menus
    miFolder = new JMenuItem("Create Folder");
    miFolder.addActionListener(this);
    //miFolder.setShortcut(new MenuShortcut(KeyEvent.VK_F));
    mFile.add(miFolder);
    mFile.addSeparator();
    miOptions = new JMenuItem("Options");
    miOptions.addActionListener(this);
    //miOptions.setShortcut(new MenuShortcut(KeyEvent.VK_O));
    mFile.add(miOptions);
    miBeanExp = new JMenuItem("BeanExplorer");
    miBeanExp.addActionListener(this);
    mFile.add(miBeanExp);
    mFile.addSeparator();
    miExit = new JMenuItem("Exit");
    miExit.addActionListener(this);
    //miExit.setShortcut(new MenuShortcut(KeyEvent.VK_E));
    mFile.add(miExit);

    miCopy = new JMenuItem("Copy");
    //miCopy.setShortcut(new MenuShortcut(KeyEvent.VK_C));
    miCopy.addActionListener(this);   
    mEdit.add(miCopy);
    miPaste = new JMenuItem("Paste");
    //miPaste.setShortcut(new MenuShortcut(KeyEvent.VK_V));
    miPaste.addActionListener(this);  
    miPaste.setEnabled(false);
    mEdit.add(miPaste);
    
    miEditBM = new JMenuItem("Edit Bookmarks");
    //miEditBM.setShortcut(new MenuShortcut(KeyEvent.VK_B));     
    miEditBM.addActionListener(this);   
    mBookmark.add(miEditBM);
    mBookmark.addSeparator();
    miRefresh = new JMenuItem("Refresh tree");
    //miRefresh.setShortcut(new MenuShortcut(KeyEvent.VK_R));     
    miRefresh.addActionListener(this);   
    mBookmark.add(miRefresh);
    
    miLSearch = new JMenuItem("local Search");
    miLSearch.addActionListener(this); 
    mSearch.add(miLSearch);
    miGSearch = new JMenuItem("Search the Web");
    miGSearch.addActionListener(this); 
    mSearch.add(miGSearch);
    
    miCLB = new JMenuItem("Create clb-files");
    miCLB.addActionListener(this); 
    mExtra.add(miCLB);
    miImages = new JMenuItem("Create images");
    miImages.addActionListener(this);
    mExtra.add(miImages);
    mExtra.addSeparator();
    miPrintURL = new JMenuItem("Print URL");
    miPrintURL.addActionListener(this);
    mExtra.add(miPrintURL);
    mExtra.addSeparator();
    miShowConsole = new JMenuItem("Show Message Window");
    miShowConsole.addActionListener(this);
    mExtra.add(miShowConsole);
    miHelp = new JMenuItem("Help");
    miHelp.addActionListener(this); 
    //miHelp.setShortcut(new MenuShortcut(KeyEvent.VK_H));    
    mHelp.add(miHelp);
    miAbout = new JMenuItem("About");
    miAbout.addActionListener(this); 
    //miAbout.setShortcut(new MenuShortcut(KeyEvent.VK_A));    
    mHelp.add(miAbout);
  }


  public void createTree() {
    if (debug) msg( "-#- createTree..." );

    tree = new JTree();

    tree.setModel( new DefaultTreeModel( new SortedTreeNode( "empty" )));
    tree.setRootVisible(false);
    tree.setLargeModel( true );
    //tree.setDoubleBuffered( true );
    // broken:
    //tree.setMinimumSize( new Dimension(100,100) );
    //tree.setPreferredSize( new Dimension(300, 600) );

    // set TreeCellRenderer (used to draw nodes)
    //
    tree.setCellRenderer(new HadesTreeCellRenderer());
    tree.putClientProperty("JTree.lineStyle", "Angled");

    // how many Components can be selected at one time?
    //
    DefaultTreeSelectionModel dtsm = new DefaultTreeSelectionModel();
    dtsm.setSelectionMode(DefaultTreeSelectionModel.SINGLE_TREE_SELECTION);
    tree.setSelectionModel(dtsm);

    treeScroller = new JScrollPane( tree );
    treeScroller.setPreferredSize( new Dimension(250,600) );
  }


  public void createTreePopupMenu() {
    if (debug) msg( "-#- createTreePopupMenu..." );

  }


  public void createImagePanel() {
    if (debug) msg( "-#- createImagePanel..." );

    imageCanvas   = new JLabel();
    imageCanvas.setIcon( 
      new ImageIcon( 
         getClass().getResource( "/hades/gui/images/lcd-matrix.gif" )
      )
    );
    imageCanvas.setBackground( Color.white );
    imageCanvas.setForeground( Color.orange );
    imageCanvas.setHorizontalAlignment( SwingConstants.CENTER );

    imageScroller = new JScrollPane( imageCanvas );
    imageScroller.setBackground( Color.white );
    imageScroller.setMinimumSize( new Dimension( 225, 100)); // initial split
    imageScroller.setPreferredSize( new Dimension(300,225));
  }


  public void createInfoPanel() {
    if (debug) msg( "-#- createInfoPanel..." );
    infoPanel = new InfoPanel();
  }
   

  public void createSplitPaneAndLayout() {
    if (debug) msg( "-#- createSplitPaneAndLayout..." ); 

    // contains all panels on the right
    //rightPanel = new JPanel();
    //rightPanel.setBackground( Color.lightGray );
    //rightPanel.setLayout( new GridLayout(2,1) );
    //rightPanel.setMinimumSize( new Dimension(10,10) );
    //rightPanel.add( imageScroller );
    //rightPanel.add( infoPanel );

    rightPanel = new JSplitPane( JSplitPane.VERTICAL_SPLIT, false,
                                 imageScroller, infoPanel );
    rightPanel.setOneTouchExpandable( true );
    rightPanel.setBackground( Color.lightGray );
    rightPanel.setMinimumSize( new Dimension( 50, 200 ));

    JPanel leftPanel = new JPanel();
    leftPanel.setLayout( new BorderLayout() );
    leftPanel.add( "Center", treeScroller );
    
    // panel that enables moving the Divider between tho contained panels
    splitPane = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT, 
                                false, leftPanel, rightPanel);
    splitPane.setDividerLocation( 200 );
    splitPane.setDoubleBuffered( true );

    this.getContentPane().add( "Center", splitPane );
    //this.setSize( 500,400 );
    //this.pack();
  }




  /**
   * try to set the Swing default look and feel 
   */
  public void setLookAndFeel() {
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      //UIManager.setLookAndFeel(
      //                     UIManager.getCrossPlatformLookAndFeelClassName());
      //UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
      //UIManager.setLookAndFeel("javax.swing.plaf.motif.MotifLookAndFeel");
      //UIManager.setLookAndFeel("javax.swing.plaf.windows.WindowsLookAndFeel");
    } 
    catch( Exception e ) {
      msg( "-W- could not set Swing look-and-feel: " + e );
    }
  }

  
  public InfoPanel getInfoPanel() {
    return infoPanel;
  }

  public JLabel getImageCanvas() {
    return imageCanvas;
  }

  public void setThumbnail( Image thumbnail ) {
    // creating an ImageIcon from an existing image is fast and cheap
    //
    ImageIcon icon; 
    if (thumbnail != null) icon = new ImageIcon( thumbnail );
    else                   icon = null;
    imageCanvas.setIcon( icon );
  }







/** Event handling for ItemEvents (Debug sub-menu) 
  public void itemStateChanged(ItemEvent e)
  {
    if (e.getSource() instanceof CheckboxMenuItem)
    {
      CheckboxMenuItem cmi = (CheckboxMenuItem)e.getSource();
      Class cl = null;
      java.lang.reflect.Field field = null;
      try
      { cl = Class.forName("hades.manager."+cmi.getLabel()); }
      catch (Exception ex)
      { message("-W- BrowserFrame.itemStateChanged("+cmi.getLabel()+") -> Class not found: hades.manager."+cmi.getLabel()); }
      try
      {
        field = cl.getDeclaredField("debug");
        field.setBoolean(null, cmi.getState());
      }
      catch (Exception ex)
      { message("-W- BrowserFrame.itemStateChanged("+cmi.getLabel()+") -> Field not found: "+field+" (Class="+cl+")"); }
    }    
  }

*/

/** Event handling for ActionEvents */
  public void actionPerformed(ActionEvent e) 
  {
    msg( "-action: " + e );
    Object source = e.getSource();

    if      (source == miAbout) showAboutDialog();
    else if (source == miHelp)  showHelp();
  }


  public void showAboutDialog() {
    ImageIcon icon = 
      new ImageIcon( ImageHelper.loadResourceImage(
        "/hades/manager/icons/colibri.gif" ));
    String messages[] = {
         "Colibri Browser", 
         " ",
         "(C) 1998-2001 by",
         "Karola Krnert",
         "Ulrich Dallmann" 
    };
    JOptionPane.showMessageDialog( this, messages, "About Colibri",
                                   JOptionPane.OK_CANCEL_OPTION,
                                   icon );
  }


  public void showHelp() {
    msg( "-W- showHelp NOT YET..." );
    //  HtmlFrame help = new HtmlFrame();
    //  help.setSize(700,550);
    //  help.setBackground(Color.white);
    //  help.show();
  }


/*
    String arg = (String) e.getActionCommand();
    // Menus 
    // Create Folder
    if (e.getSource() == miFolder)
    {
      insertFolder();
    }
    // Options
    if (e.getSource() == miOptions)
    {
      Options.getOptions().editOptions(this);
    }
    // BeanExplorer
    if (e.getSource() == miBeanExp)
    {
      BeanExplorer bex = new BeanExplorer();
      bex.setSize(600,450);
      bex.show();
    }
    // Exit
    else if (e.getSource() == miExit)
    {
      setVisible(false);    
      if (browser.isMain()) System.exit(0);
    }
    // Copy
    else if (e.getSource() == miCopy)
    {
      clipboard = getSelected();
      message("Copy: "+clipboard);
      miPaste.setEnabled(true);
    }
    // Paste
    else if (e.getSource() == miPaste)
    {
      if (clipboard != null)
      {
        TNode curDir = getSelected();
        // if curDir is a component paste in to the parent directory
        if (curDir.getUserObject() instanceof ComponentInfo) 
          curDir = (TNode)curDir.getParent();
        // create a new TNode for the copy, UserObject will be added later
        TNode copy = new TNode(null, curDir);
        copyComponent((TNode)clipboard, copy);  
      }
    }
    // Edit Bookmarks
    else if (e.getSource() == miEditBM)
    {
      Options.getOptions().editBookmarks(this);
    }
    // Refresh tree
    else if (e.getSource() == miRefresh)
    {
      treePane.updateTree();
    }
    // Search
    else if (e.getSource() == miLSearch)
    {
      search();
    }
    // Search
    else if (e.getSource() == miGSearch)
    {
      searchServer();
    }
    // Create clb
    else if (e.getSource() == miCLB)
    {
      String name = JOptionPane.showInputDialog
        ("Please select the directory you wish to import:");       
      message("name = "+name);
      ImportCommand ic = new ImportCommand(this);
      ic.importDir(name);
    }
    // Create images
    else if (e.getSource() == miImages)
    {
      String name = JOptionPane.showInputDialog
        ("Please select the directory for which you wish to create images:");       
      ImportCommand ic = new ImportCommand(this);
      ic.createImages("", name);
    }
    // Print URL
    else if (e.getSource() == miPrintURL)
    {
      TNode tnode = (TNode)getSelected();
      message("Print URL:");
      message("   node-URL: "+ getURLStringForNode(tnode));
      if (tnode.getUserObject() instanceof ComponentInfo)
      {
        ComponentInfo clb = (ComponentInfo)tnode.getUserObject();
        message("   clb-URL: "+ clb.getURL());
        message("   ClassLoader: "+ browser.getColibriLoader(getURLStringForNode(tnode)));
      }
    }
    else if (e.getSource() == miShowConsole)
    {
      hades.manager.Console.getConsole().setVisible(true);
    }
    // Help
    // About
    else if (e.getSource() == miAbout)
    {
      About about = new About(this);
      about.setSize(300,300);
      about.setBackground(Color.white);
      about.show();
    }
    // Popup-Menues
    // Popup-Menue for components
    // Properties
    else if (e.getSource() == popprop)
    {
      TNode selected = (TNode)getSelected();
      // ComponentInfo
      if (selected!=null && selected.getUserObject() instanceof ComponentInfo) 
      {
        TNode tnode = selected.getTop();
        if (tnode.getUserObject() instanceof Bookmark)
          ((ComponentInfo)selected.getUserObject()).showDialog(this, true, false);
        else 
        {
          ComponentInfo clb = (ComponentInfo)selected.getUserObject();
          clb.showDialog(this, true, true);
          infoPanel.setInfo(clb);
          imageCanvas.setImage(clb.getImageObject());
        }
      }
    }
    // Delete
    else if (e.getSource() == popdelc)
    {
      TNode selected = (TNode)getSelected();
      if (selected!=null) 
      {
        Object[] options = {"Delete", "Cancel"};
        int erg = JOptionPane.showOptionDialog(this,"Are you sure, you want to delete the component and all its files ?", "Delete Component", JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, null, options, options[0]);
        this.toFront();
        if (erg == 0) deleteNode(selected);
      }
    }
    // jar
    else if (e.getSource() == popjar)
    {
      TNode selected = (TNode)getSelected();
      if (selected!=null) 
      {
        String urlstring = getURLStringForNode(selected);
        ComponentInfo clb = (ComponentInfo)selected.getUserObject();
        SaveCommand sc = new SaveCommand();
        //sc.createJar(clb);
        //message("*******************************************");
        sc.createJar(urlstring);
      }
    }
    // html
    else if (e.getSource() == pophtml)
    {
      TNode selected = (TNode)getSelected();
      if (selected!=null) 
      {
        ComponentInfo clb = (ComponentInfo)selected.getUserObject();      
        TNode tnode = selected.getTop();
        String urlstring="";
        if (tnode.getUserObject() instanceof Bookmark)
          urlstring =  "http://"+clb.getServer()+"/"+clb.getRelativePath()+clb.getHighestVersion()+"/"+clb.getHTML();
        else if (tnode.getUserObject() instanceof String) 
        {
          if (tnode.getUserObject().toString().equals("local"))
          {
            urlstring =  "file:"+Options.getOptions().getLocalDir()+"/"+clb.getRelativePath()+clb.getHighestVersion()+"/"+clb.getHTML();
          }
          else
          {
            urlstring =  "file:"+Options.getOptions().getSysDir()+"/"+clb.getRelativePath()+clb.getHighestVersion()+"/"+clb.getHTML();
          }
        }
        message(urlstring);
        HtmlFrame help = new HtmlFrame(urlstring);
        help.setSize(700,550);
        help.setBackground(Color.white);
        help.show();
      }
    }
    // Open
    else if (e.getSource() == popopen)
    {
      TNode selected = (TNode)getSelected();
      if (selected!=null) 
      {
        opened(selected);
      }
    }
    // Popup-Menue directories
    // Update
    else if (e.getSource() == popupdated)
    {
      TNode selected = (TNode)getSelected();
      if (selected!=null) 
      {
        treePane.updateTree(selected);
      }
    }
    // Delete
    else if (e.getSource() == popdeld)
    {
      TNode selected = (TNode)getSelected();
      if (selected!=null) 
      {
        Object[] options = {"Delete", "Cancel"};
        int erg = JOptionPane.showOptionDialog
          (this,"Are you sure, you want to delete the directory and all its files ?",
          "Delete Component", JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, 
          null, options, options[0]);
        this.toFront();
        if (erg == 0) deleteNode(selected);
      }
    }
    // Create subfolder
    else if (e.getSource() == popfolderd)
    {
      insertFolder();
    }
    
    // Popup-Menue bookmarks
    // Update
    else if (e.getSource() == popupdateb)
    {
      TNode selected = (TNode)getSelected();
      if (selected!=null) 
      {
        treePane.updateTree(selected);
      }
    }
    // Remove
    else if (e.getSource() == poprem)
    {
      TNode selected = (TNode)getSelected();
      // local and system can not be removed!
      // Bookmark
      if ((selected!=null)&&(selected.getUserObject() instanceof Bookmark))
      {
        Object[] options = {"Delete", "Cancel"};
        int erg = JOptionPane.showOptionDialog
          (this,"Are you sure, you want to remove the bookmark?", 
          "Delete Bookmark", JOptionPane.DEFAULT_OPTION, 
          JOptionPane.WARNING_MESSAGE, null, options, options[0]);
        this.toFront();
        if (erg == 0)
        {
          Bookmark bm = (Bookmark)selected.getUserObject();
          if (Options.getOptions().getBookmarks().removeElement(bm))
            removeFromTree(bm);
          Options.getOptions().saveOptions();
        }
      }
      // SearchResult
      else if ((selected!=null)&&(selected.getUserObject() instanceof String) 
        && selected.getUserObject().toString().startsWith("search"))
      {
        TNode parent = (TNode)selected.getParent();
        parent.remove(selected);
        ((DefaultTreeModel)treePane.getTree().getModel()).nodeStructureChanged(parent);
      }
    }
    // Create subfolder
    else if (e.getSource() == popfolderb)
    {
      insertFolder();
    }
  }
*/
  
  
 
  /* ********************************************************************* */
  /* ********************************************************************* */ 
  /*     Event Listeners                                                   */
  /* ********************************************************************* */ 
  /* ********************************************************************* */ 

  
/** send the componentSelected event to the listener 
  public void selected(TNode component)
  {
    if (component.getUserObject() instanceof ComponentInfo)
    {
      String urlstring = getURLStringForNode(component);
      if (debug) message("-I- Browser.selected() url: "+urlstring);
      SelectEvent evt = new SelectEvent(urlstring, SelectEvent.COMPONENT_SELECTED);
      browser.selected(evt);
    }
  }
*/
  
/** send the componentSelected event to the listener 
  public void opened(TNode component)
  {
    if (component.getUserObject() instanceof ComponentInfo)
    {
      String urlstring = getURLStringForNode(component);
      SelectEvent evt = new SelectEvent(urlstring, SelectEvent.COMPONENT_SELECTED);
      browser.opened(evt);
    }
  }
*/
  
  /**
  *  will create the URLstring for a node in the tree
  *
  private String getURLStringForNode(TNode tnode)
  {
    String path = tnode.getPath();
    TNode top = tnode.getTop();
    String urlstring = null;
    if (top.getUserObject() instanceof Bookmark)
      urlstring = "clb://"+((Bookmark)top.getUserObject()).getHost()+path;
    else
      urlstring = "clb://local"+path;
    if (tnode.getUserObject() instanceof ComponentInfo)
    {
      if (infoPanel.getVersion()!=null)
        urlstring += "?version=" + infoPanel.getVersion();
    }
    if (debug) message("-I- BrowserFrame.getURLStringForPath() -> "+urlstring);
    return urlstring;
  }
  */  



  
  
  
  
  /**************************************************************/ 
  /* event handlers                                             */
  /**************************************************************/ 
 


  /**
   * react to tree selection events: 
   * for SimObjects, display their symbol and CLB component info,
   * for Designs, display their thumbnail and CLB info,
   * for README files, display their text,
   * do nothing for other data types (yet).
   */
  class TreeSelectionHandler implements TreeSelectionListener {

    public void valueChanged( TreeSelectionEvent e ) {
      //msg( "-#- TreeSelectionHandler.valueChanged: " + e );

      SortedTreeNode node = null;
      try {
        node = (SortedTreeNode) e.getPath().getLastPathComponent();
        node.handleTreeSelection( BrowserFrame.this ); 
      }
      catch( Exception ee ) {
        msg( "-E- internal: " + ee );
        ee.printStackTrace();
      }
    } // valueChanged

  } // TreeSelectionHandler

/*
        msg( "-S- " + node );
        if (node instanceof SimObjectTreeNode) {
          // infoPanel.setInfo(ci); FIXME
          imageCanvas.setIcon(
            new ImageIcon( 
              DesignManager.getDesignManager().getThumbnail( 
                treeManager.getFullPathname( node ))));
          imageScroller.getViewport().setViewPosition( new Point(0,0) );
          infoPanel.setInfoText( "" );
          infoPanel.setName( "" );
          infoPanel.setAuthor( "" );
          infoPanel.setVersion( "" );
        }
        else if (node instanceof DesignTreeNode) {
          // infoPanel.setInfo(ci); FIXME
          imageCanvas.setIcon( 
            new ImageIcon( 
              DesignManager.getDesignManager().getThumbnail( 
                treeManager.getFullPathname( node ))));
          imageScroller.getViewport().setViewPosition( new Point(0,0) );
        }
        else if (node instanceof ReadmeTreeNode) {
          // infoPanel.setInfo(ci); FIXME
msg( "-#- ...loading README..." );
          String info = DesignManager.getDesignManager().readTextFile(
                          treeManager.getFullPathname( node ));
          infoPanel.setInfoText( info );
          infoPanel.setName( "README" );
          infoPanel.setAuthor( "-" );
          infoPanel.setVersion( "-" );
          imageCanvas.setIcon( null );
        }
        else { // other object, no thumbnail
          imageCanvas.setIcon( null);
        }
      }
*/

 

 
  /** 
   * handle window events: a JFrame already knows to setVisible(false)
   * but we still may want to exit... (not yet)
   */
  class WindowHandler extends WindowAdapter 
  {
    public void windowClosing(WindowEvent e) 
    {
      setVisible(false);
      //if (browser.isMain()) System.exit(0); FIXME
    }
  }

  
  /**************************************************************/ 
  /* utitity methods                             */
  /**************************************************************/ 
  
  /** sets the Icon in the top left corner of the Browser-Window */
  private void getIcon() {
    Image icon = null;
    try {
      java.net.URL url = getClass().getResource( 
        "/hades/manager/images/iconManager.gif" );
      icon = Toolkit.getDefaultToolkit().getImage( url );
      this.setIconImage( icon );
    }
    catch( Exception e ) {
      msg( "-E- EditFrame.getIcon(): got Exception " + e );
      msg( "... you won't see the HADES manager icon." );
    }
  }




  /** send a message to the Console */
  public static void msg( String s ) { 
    System.out.println( s );
    //hades.gui.Console.getConsole().message( msg );
  }
 
 
  public String toString() {
    return versionString;
  }


  public static void main( String argv[] ) throws Exception {

    TreeManager tb      = new TreeManager();
    tb.setRootNode( new RootTreeNode() );

    tb.registerZipFile( "archives/t1-libraries.jar" );
    tb.registerZipFile( "archives/t3-vorlagen.jar" );

    SortedTreeNode pwd = new FileRootTreeNode();
    pwd.setUserObject( "file://" + "." ); // FIXME
    tb.getRootNode().add( pwd );
    tb.testFileSystem( ".", pwd );


    BrowserFrame bframe = new BrowserFrame();
    bframe.setTreeManager( tb );

    bframe.tree.setModel( new DefaultTreeModel( tb.getRootNode() ));
    bframe.setSize( 750, 600 );
    bframe.show();
  }


}
