/* DesignFileTreeNode - hades.manager.tree.DesignFileTreeNode
 *
 * node for a Hades Design in the Colibri tree
 *
 * 04.03.01 - handleTreeSelection, first attempt at handleDoubleClick
 * 01.03.01 - first version
 */

package hades.manager.tree;

import javax.swing.ImageIcon;
import hades.manager.*;
import hades.models.Design;


public class  DesignFileTreeNode
       extends  SortedTreeNode {


  static ImageIcon _icon = loadIcon( "/hades/manager/icons/design.gif" );

  public ImageIcon getIcon() {
    return _icon;
  }


  /**
   * load and set the thumbnail Icon for this HADES design,
   * also try to * load and display the name/author/version/description 
   * attributes
   * and display in the Browser InfoPanel
   */
  public void handleTreeSelection( BrowserFrame browser ) {
    System.out.println( "-#- DesignFileTN.handleTreeSelection: " + toString() );

    String pathname = TreeManager.getFullPathname( this );
    String thumbname= DesignManager.changeExtension( pathname, ".gif" );

    browser.setThumbnail(
              DesignManager.getDesignManager().getThumbnail( thumbname ));

    // FIXME: should load and parse ComponentInfo CLB
    InfoPanel ip = browser.getInfoPanel();
    ip.setName( toString() );
    ip.setAuthor( "-" );
    ip.setVersion( "-" );
    ip.setInfoText( "" );
  }




  /**
   * handle a tree double click event on this node in the Colibri tree control.
   * For a Hades design, we open the design in the active editor.
   */
  public void handleDoubleClick( BrowserFrame browser ) {
    msg( "-#- DesignFileTN.handleDoubleClick ignored: " + toString() );

    String fullPathname = TreeManager.getFullPathname( this );

    hades.gui.Editor editor;
    editor = hades.gui.Editor.getActiveEditor();
    if (editor == null) editor  = new hades.gui.Editor();

// FIXME: support all design file types, not only file://
if (fullPathname.startsWith( "file://" )) {
    String filename = fullPathname.substring( 7 );
    editor.doOpenDesign( filename, true ); // check unsaved changes
}
else {
    msg( "-#- cannot handle: " + fullPathname );
    Design design = null;
    try {
      design = DesignManager.getDesignManager().
                 getDesign( editor, fullPathname, true /*toplevel*/ );
      if (design != null) editor.doOpenDesign( design, true );
    }
    catch( Exception e ) {
e.printStackTrace();
    }
}
  }
  

}
