/* ZipRootTreeNode - hades.manager.tree.ZipRootTreeNode
 *
 * the root node for a JAR/ZIP archive (read only) in Colibri
 *
 * 06.03.01 - override toString() for more readable display in the tree
 * 01.03.01 - first version
 */

package hades.manager.tree;

import java.util.zip.ZipFile;
import javax.swing.ImageIcon;


/**
 * ZipRootTreeNode: a tree node to represent a single JAR/ZIP file root. 
 * An application creating a ZipRootTreeNode is expected to also
 * call setZipFile( someZipFile ), from which the tree node 
 * constructs a "zip://" + someZipFile.getName() + "?" pathname.
 */
public class  ZipRootTreeNode
       extends  SortedTreeNode {

  ZipFile zipfile = null;

  static ImageIcon _icon = loadIcon( "/hades/manager/icons/jarzip.gif" );

  public ImageIcon getIcon() {
    return _icon;
  }


  public void setZipFile( ZipFile zipfile ) {
    this.zipfile = zipfile;
    setUserObject( "zip://" + zipfile.getName() + "?" );
  }

  public ZipFile getZipFile() {
    return zipfile;
  }


  public String toString() {
    if (zipfile == null) return "null";
  
    String s = zipfile.getName();
    int  pos = s.lastIndexOf( java.io.File.separatorChar );
    if ((pos >= 0) && pos < s.length()) return s.substring( pos+1 );
    else                                return s;
  }

  
}
